<?php


define( "REL_DIR_CONTAINS", 0);
define( "REL_DIR_DEPENDS", 1);
define( "REL_DIR_REF", 0);

define( "REL_TYPE_STRONG", 0x01);
define( "REL_TYPE_WEAK", 0x02);
define( "REL_TYPE_SINGLE", 0x04);


define( "REL_LINK_CREATE", 0x01); 
define( "REL_LINK_SEARCH", 0x02);
define( "REL_LINK_OPTIONAL", 0x04);
define( "REL_LINK_ROOT", 0x08);

define( "REL_LINK_NONE", 0x00); 
define( "REL_LINK_RELATEED", REL_LINK_CREATE ); 
define( "REL_LINK_LINKED_CREATE", REL_LINK_CREATE | REL_LINK_SEARCH ); 
define( "REL_LINK_LINKED_QUERY", REL_LINK_SEARCH ); 
define( "REL_LINK_LINKED_OPTIONAL", REL_LINK_SEARCH | REL_LINK_OPTIONAL ); 

class RelationNodeObject extends NodeObject {

 function RelationNodeObject() {

 }
 

 function getSearchKey() { return false; }

 function searchObjectId($key) { return 0; }

 function getRelations() { return array(); }

 function getRelation($name) { return array(); }
 
}

class RelationDefinition extends BaseObject {

 var $DIR_CONTAINS = REL_DIR_CONTAINS; 
 var $DIR_DEPENDS = REL_DIR_DEPENDS; 
 var $DIR_REF = REL_DIR_REF; 

 var $TYPE_STRONG = REL_TYPE_STRONG;
 var $TYPE_WEAK = REL_TYPE_WEAK;
 
 var $name;
 var $dir;
 var $optional;
 var $type;
 var $objectTypes;
 
 function RelationDefinition($name, $dir, $optional, $type, $objectTypes = false) {
 $this->name = $name;
 $this->dir = $dir;
 $this->optional = $optional;
 $this->type = $type;
 $this->objectTypes = $objectTypes;
 }

 function getName() {
 return $this->name;
 }

 function getDirection() {
 return $this->dir;
 }

 function isOptional() {
 return $this->optional;
 }

 function getType() {
 return $this->type;
 }
 
 function getObjectTypes() {
 return $this->objectTypes;
 }
}

class RelationObject {
 
 var $id;
 var $type;
 var $object;
 
 function RelationObject($id, $type = 0) {
 
 if (is_object($id)) {
 $this->object = $id;
 $this->id = $id->getId();
 $this->type = $id->getTType();
 } else {
 $this->id = $id;
 $this->type = $type;
 $this->object = null;
 }
 }

 function getId() {
 return $this->id;
 }

 function getType() {
 return $this->type;
 }

 function &getNodeObject() {
 if ($this->object === null) {
 $this->object =& NodeFactory::getInstanceOf($this->type, $this->id);
 }
 return $this->object;
 }

}

class RelationContainer {

 var $relObject;
 var $mappedId;
 var $relations;
 var $linkType;

 function RelationContainer(&$relObject, $linkType) {
 $this->relObject =& $relObject;
 $this->linkType = $linkType;
 $this->mappedId = 0;
 $this->relations = array();
 }
 
 function getId() {
 return $this->relObject->getId();
 }
 
 function getType() {
 return $this->relObject->getType();
 }
 
 function &getObject() {
 return ($this->relObject->getNodeObject());
 }
 
 function getMappedId() {
 return $this->mappedId;
 }

 function setMappedId($id) {
 $this->mappedId = $id;
 }

 function addRelation($name, $type, $id, $linkType) {
 $this->relations[$name][$type][$id] = $linkType;
 }
 
 function setLinkType($type) {
 $this->linkType = $type;
 }
 
 function getLinkType() {
 return $this->linkType;
 }
 
 function exportXml($indent) {

 $rObj =& $this->getObject();
 
 $xml = "";

 $xml .= "$indent<object>\n";
 $xml .= "$indent\t<id>".$this->getMappedId()."</id>\n";
 $xml .= "$indent\t<type>".$this->getType()."</type>\n";
 $xml .= "$indent\t<linktype>".$this->linkType."</linktype>\n";
 $xml .= $rObj->exportXml("$indent\t");
 $xml .= "$indent\t<relations>\n";
 
 foreach ($this->relations as $name=>$rel) {
 $xml .= "$indent\t\t<relation name=\"$name\">\n";
 
 foreach ($rel as $type=>$val) {
 foreach ($val as $id=>$linkType) {
 $xml .= "$indent\t\t\t<rel type=\"$type\">$id</rel>\n";
 }
 }
 
 $xml .= "$indent\t\t</relation>\n";
 }
 
 $xml .= "$indent\t</relations>\n";
 $xml .= "$indent</object>\n";
 
 return $xml;
 }

 function importXmlStruct($struct) {

 }
}

class RelationProcessor {

}

class XmlRelationProcessor extends RelationProcessor {

 var $xml;
 var $indent;
 
 function XmlRelationProcessor($indent) {
 $this->xml = "";
 $this->indent = $indent;
 }

 function getXml() {
 return $this->xml;
 }

 function processElement(&$relContainer) {
 
 $this->xml .= $relContainer->exportXml($this->indent);
 }
 
}

class RelationObjectList extends BaseObject {

 var $d_unprocessed = array();
 var $d_root = array();
 
 var $d_idMap = array();
 var $d_origIdMap = array();
 var $d_maxMapId = array();
 var $d_linkTypes = array();

 function RelationObjectList($rootObjs = false) { 
 
 if (is_array($rootObjs)) {
 foreach ($rootObjs as $key=>$obj) {
 $this->addRootObject($obj);
 }
 } else if (is_object($rootObjs)) {
 $this->addRootObject($rootObjs);
 }
 
 }

 function addRootObject(&$obj) {
 $this->d_root[] = new RelationContainer(new RelationObject($obj), REL_LINK_CREATE | REL_LINK_ROOT );
 }

 function &getRootObjects() {
 
 }

 function processObjects(&$processor) {

 $this->_resetQueues();
 
 foreach ($this->d_root as $key=>$root) {
 $this->_queueRelation($this->d_root[$key]);
 }

 unset($rel);
 while ($rel =& $this->_getNextRelation()) {
 
 unset($rObj);
 $rObj =& $rel->getObject();

 $relDefs = $rObj->getRelations();
 foreach ($relDefs as $relDef) {
 
 $linkType = $this->_getRelationType($relDef);

 if ($linkType == REL_LINK_NONE) continue;

 unset($rels);
 $rels =& $rObj->getRelation($relDef->getName());

 foreach ($rels as $key=>$r) {
 
 unset($relObj);
 $relObj =& $rels[$key];
 
 $mId = $this->_getProcessedMappedId($relObj, $linkType);

 if (!$mId) {
 $mId = $this->_queueRelation(new RelationContainer($relObj, $linkType));
 }

 $rel->addRelation($relDef->getName(), $relObj->getType(), $mId, $linkType);
 }
 
 }
 
 $processor->processElement($rel);

 $this->_finishRelation($rel);
 
 unset($rel);
 }
 
 }
 
 function _getProcessedMappedId(&$relObject, $linkType) {
 $oType = $relObject->getType();
 $mId = $this->_getObjMappedId($relObject->getNodeObject());
 
 $type = $this->d_linkTypes[$oType][$mId];
 return ($type === null || (($type & REL_LINK_LINKED > 0) && ($linkType & REL_LINK_RELATED > 0))) ? 0 : $mId;
 }

 function _getRelationType($relDef) {
 $linkType = REL_LINK_NONE;
 
 if ($relDef->getType() & REL_TYPE_STRONG > 0) {
 
 $linkType = $relDef->getDirection() == REL_DIR_CONTAINS ? REL_LINK_RELATED : REL_LINK_LINKED;
 if ($relDef->isOptional()) $linkType |= REL_LINK_OPTIONAL;
 
 } else if ($relDef->getType() & REL_TYPE_WEAK > 0) {
 
 $linkType = REL_TYPE_LINKED;
 if ($relDef->isOptional()) $linkType |= REL_LINK_OPTIONAL;
 
 }
 
 return $linkType;
 }
 
 function _resetQueues() {
 $this->d_unprocessed = array();
 $this->d_linkTypes = array();
 }
 
 function _queueRelation(&$relContainer) {
 
 $mId = $this->_getObjMappedId($relContainer->getObject());

 $relContainer->setMappedId($mId);
 
 $this->d_unprocessed[] =& $relContainer;
 
 return $mId;
 }

 function _finishRelation(&$relContainer) {
 $this->d_linkTypes[$relContainer->getType()][$relContainer->getMappedId()] = $relContainer->getLinkType();
 }
 
 function &_getNextRelation() {
 if (count($this->d_unprocessed) == 0) return null;

 reset($this->d_unprocessed);
 list($key, $rel) = each($this->d_unprocessed);

 unset($rel);
 $rel =& $this->d_unprocessed[$key];
 
 unset($this->d_unprocessed[$key]);
 
 return $rel;
 }

 function _getObjMappedId(&$obj) {
 
 return $this->_getMappedId($obj->getTType(), $obj->getID());
 }
 
 function _getMappedId($type, $id) {
 if ($id == 0) {
 $mId = $this->_createMapId($type);
 $this->d_origMapId[$type][$mId] = $id;
 return $mId;
 }
 
 if (!$this->d_idMap[$type][$id]) {
 $mId = $this->_createMapId($type);
 $this->d_idMap[$type][$id] = $mId;
 $this->d_origMapId[$type][$mId] = $id;
 }
 return $this->d_idMap[$type][$id];
 }

 function _createMapId($type) {
 $max = $this->d_maxMapId[$type];
 if (!$max) $max = 0;
 $id = $max + 1;
 $this->d_maxMapId[$type] = $id;
 return $id;
 }

 function _setOrigId($type, $mappedId, $id) {
 if ($id > 0) $this->d_idMap[$type][$id] = $mappedId;
 $this->d_origMapId[$type][$mappedId] = $id;
 }
 
 function _getOrigId($type, $mappedId) {
 $id = $this->d_origMapId[$type][$mappeId];
 return $id ? $id : 0;
 }
}


class ExportController extends BaseObject {

 function ExportController() {

 }

 function exportObject($obj) {
 return $this->exportObjects(array($obj));
 }
 
 function exportObjects($objs) {

 $ex = new RelationObjectList($objs);

 $processor = new XmlRelationProcessor("\t");
 
 $ex->processObjects($processor);

 
 node_include("xml");
 $xmlParser = new XMLParser();

 $xml = $xmlParser->getHeader()."\n";

 $xml .= "<objects>\n";
 
 $xml .= $processor->getXml();
 
 $xml .= "</objects>\n";

 return $xml;
 }

 function &importObjects($xml) {
 
 node_include("xml");
 $xmlParser = new XMLParser();

 $xmlParser->loadXML($xml);
 $struct = $xmlParser->getStruct();
 

 $ex = new RelationObjectList();

 

 return $ex->getRootObjects();
 }
}

?>
