<?php

class ChartComponent extends FormComponent {

 var $d_chartRenderer;
 var $d_border;

 function ChartComponent(&$chartRenderer, $label = NULL, $border = 0, $editable = false) {
 parent::FormComponent($label, $editable, true);
 $this->d_chartRenderer =& $chartRenderer;
 $this->d_border = $border;
 }

 function getSerializeKey() {
 $rs = array();
 $rs["parent"] = parent::getSerializeKey();
 $rs["border"] = $this->d_border;
 $rs["renderer"] = node_serialize($this->d_chartRenderer);
 return $rs;
 }
 
 function initFromSerializeKey($key) {
 parent::initFromSerializeKey($key["parent"]);
 $this->d_chartRenderer =& node_unserialize($key["renderer"]);
 $this->d_border = $key["border"];
 }

 function getHTML( &$form, $fieldPrefix, $doCheck, $checkSeverity, $key ) {
 $html = "";
 if ($this->d_border) $html .= "<table cellspacing=0 cellpadding=1 border=0 bgcolor=#000000><tr><td>";
 $html .= $this->d_chartRenderer->getHtmlOutput();
 if ($this->d_border) $html .= "</td></tr></table>";
 return $this->mergeResult( parent::getHTML( $form, $fieldPrefix, $doCheck, $checkSeverity, $key ), $html );
 }
}

class ChartsComponent extends CollectionComponent {

 var $d_chartData;
 var $d_charts;
 var $d_allCharts;
 var $d_activeTab;

 function ChartsComponent(&$chartData, $label = NULL) {
 parent::CollectionComponent($label);
 $this->d_allCharts = true;
 $this->d_charts = array();
 $this->d_chartData =& $chartData;
 $this->d_activeTab = false;
 $this->setComponent("t", new TabComponent(TABMODE_TRUE, false) );
 }

 function &getChartData() {
 return $this->d_chartData;
 }

 function enableAllCharts($value = true) {
 $this->d_allCharts = ($value)?true:false;
 }

 function enableChart($type) {
 $this->d_charts[$type] = true;
 }

 function disableChart($type) {
 $this->d_charts[$type] = false;
 }

 function setChartRenderer($type, &$dataRenderer) {
 $this->d_charts[$type] =& $dataRenderer;
 }

 function setActiveTab($type) {
 $this->d_activeTab = $type;
 }

 function getActiveTab() {
 if ($this->d_activeTab) {
 return $this->d_activeTab;
 } else {
 $tab =& $this->getComponent("t");
 return $tab->getActiveTabName();
 }
 }

 function &getChartRenderers() {
 node_class("chartrendererfactory");
 
 if ($this->d_allCharts) {
 $charts = ChartRendererFactory::getCharts();
 } else {
 $charts = $this->d_charts;
 }
 
 $rs = array();
 foreach ($charts as $type=>$val) {
 if (isset($this->d_charts[$type]) && !is_object($this->d_charts[$type]) && $this->d_charts[$type] === FALSE ) continue;
 unset($renderer);
 if (is_object($this->d_charts[$type])) {
 $renderer =& $this->d_charts[$type];
 } else {
 $renderer =& ChartRendererFactory::getInstanceOf($type, $this->d_chartData);
 if (!is_a($renderer, "chartrenderer")) continue;
 }
 $rs[$type] =& $renderer;
 }
 
 return $rs;
 }

 function getSerializeKey() {
 $rs = array();
 $rs["parent"] = parent::getSerializeKey();
 $rs["charts"] = node_serialize($this->d_charts);
 $rs["allcharts"] = $this->d_allCharts;
 $rs["chartdata"] = node_serialize($this->d_chartData);
 $rs["activeTab"] = $this->d_activeTab;
 return $rs;
 }

 function initFromSerializeKey($key) {
 parent::initFromSerializeKey($key["parent"]);
 $this->d_charts = node_unserialize($key["charts"]);
 $this->d_allCharts = $key["allcharts"];
 $this->d_chartData = node_unserialize($key["chartdata"]);
 $this->d_activeTab = $key["activeTab"];
 }

 function getHTML( &$form, $fieldPrefix, $doCheck, $checkSeverity, $key ) {
 $charts =& $this->getChartRenderers();
 
 $tab =& $this->getComponent("t");
 $tabs = $tab->getTabs();
 $ctabs = array();
 
 
 foreach ($charts as $type=>$renderer) {
 $name = ChartRendererFactory::getChartName($type);
 $tab->setTab($type, $name, new ChartComponent( $charts[$type], NULL, 0 ), $charts[$type]->getIcon() );
 $ctabs[$type] = true;
 }

 foreach ($tabs as $type=>$renderer) {
 if (!$ctabs[$type]) $tab->removeTab($type);
 }
 
 if ($this->d_activeTab) {
 $tab->setActiveTab($this->d_activeTab);
 $this->d_activeTab = false;
 }
 
 $html = $this->_getComponentHTMLTable( "t", $form, $prefix, $doCheck, $checkSeverity, $key );
 
 return $this->mergeResult( parent::getHTML( $form, $fieldPrefix, $doCheck, $checkSeverity, $key ), $html );
 }

}

?>
